#include <stdlib.h>
#include "SDL/SDL.h"
// *** IF USING XCODE ON MACOS X, YOU MAY NEED TO CHANGE THE FOLLOWING LINE TO: #include "SDL_mixer/SDL_mixer.h"
#include "SDL/SDL_mixer.h"


// --------------------------------------------------------
//  struct
// --------------------------------------------------------
struct stContext {
    int musicPlaying;
    Mix_Music* music;
    SDL_Surface *screen; //Pointer to the main screen surface
};

typedef struct stContext Context;


// --------------------------------------------------------
//  constructors
// --------------------------------------------------------
Context* init() {
    Context* ob = (Context*) malloc(sizeof(Context));
    ob->musicPlaying = 0;
    ob->music = NULL;
    ob->screen = NULL;
    return ob;
}


// --------------------------------------------------------
//  static
// --------------------------------------------------------
//
// declared static so that onMusicFinished will be able to reach it,
// due to limitation in the way arg passing works in SDL mixer API.
static Context* CONTEXT;


// --------------------------------------------------------
//  sdl mixer
// --------------------------------------------------------
void onMusicFinished()
{
    printf("onMusicFinished\n");
    CONTEXT->musicPlaying = 0;
}

int start_audio(Context* context)
{
    int audio_rate = 22050; //Frequency of audio playback
    Uint16 audio_format = AUDIO_S16SYS; //Format of the audio we're playing
    int audio_channels = 2; //2 channels = stereo
    int audio_buffers = 4096; //Size of the audio buffers in memory

    printf("Start audio\n");

    //Initialize BOTH SDL video and SDL audio
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) != 0)
    {
        printf("Unable to initialize SDL: %s\n", SDL_GetError());
        return 1;
    }

    //Initialize SDL_mixer with our chosen audio settings
    if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) != 0)
    {
        printf("Unable to initialize audio: %s\n", Mix_GetError());
        return 1;
    }

    //Load our music file from disk
    context->music = Mix_LoadMUS("music.mp3");
    if(context->music == NULL) 
    {
        printf("Unable to load music file: %s\n", Mix_GetError());
        return 1;
    }

    if(Mix_PlayMusic(context->music, 0) == -1)
    {
        printf("Unable to play music file: %s\n", Mix_GetError());
        return 1;
    }

    context->musicPlaying = 1;

    Mix_HookMusicFinished(onMusicFinished);

    return 0;
}

int stop_audio(Context* context) {
    printf("stop_audio\n");

    Mix_HaltMusic();
    Mix_FreeMusic(context->music);
    Mix_CloseAudio();

    SDL_Quit();

    return 0;
}


// --------------------------------------------------------
//  bootstrap
// --------------------------------------------------------
int create_container_screen(Context* context) {
    printf("create_container_screen\n");
    context->screen = SDL_SetVideoMode(320, 240, 0, SDL_ANYFORMAT);
    if (context->screen == NULL)
    {
        printf("Unable to set video mode: %s\n", SDL_GetError());
        return 1;
    }
    return 0;
}

void exit_on_errcode(int i) {
    if (i) {
        exit(i);
    }
}

int main() {
    Context* context;
    int errcode;

    CONTEXT = init();
    context = CONTEXT;

    errcode = create_container_screen(context);
    exit_on_errcode(errcode);

    errcode = start_audio(context);
    exit_on_errcode(errcode);

    while(context->musicPlaying)
    {
        SDL_Delay(100);
    }

    errcode = stop_audio(context);

    return 0;
}

